@@ -4,6 +4,7 @@
  * SPDX-License-Identifier: LGPL-2.0-or-later
  *
  * SPDX-FileCopyrightText: 2017 Jan Grulich <jgrulich@redhat.com>
+ * SPDX-FileCopyrightText: 2022 Harald Sitter <sitter@kde.org>
  */
 
 #ifndef XDG_DESKTOP_PORTAL_KDE_REQUEST_H
@@ -12,16 +13,26 @@
 #include <QDBusVirtualObject>
 #include <QObject>
 
+class QDBusObjectPath;
+
 class Request : public QDBusVirtualObject
 {
     Q_OBJECT
 public:
-    explicit Request(QObject *parent = nullptr, const QString &portalName = QString(), const QVariant &data = QVariant());
+    explicit Request(const QDBusObjectPath &handle, QObject *parent = nullptr, const QString &portalName = QString(), const QVariant &data = QVariant());
     ~Request() override;
 
     bool handleMessage(const QDBusMessage &message, const QDBusConnection &connection) override;
     QString introspect(const QString &path) const override;
 
+    template<class T>
+    static Request *makeClosableDialogRequest(const QDBusObjectPath &handle, T *dialogAndParent)
+    {
+        auto request = new Request(handle, dialogAndParent);
+        connect(request, &Request::closeRequested, dialogAndParent, &T::reject);
+        return request;
+    }
+
 Q_SIGNALS:
     void closeRequested();
 
